window.Gleam.initSearch([{"doc":"formz","title":"formz","content":"# formz\n\n[![Package Version](https://img.shields.io/hexpm/v/formz)](https://hex.pm/packages/formz)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/formz/)\n\nA Gleam library for parsing and generating accessible HTML forms.\n\n&gt; **Note:** This library currently has two non-interoperable ways to define forms,\none using the builder pattern, and one using a series of `use` calls like with\nthe [toy](https://hexdocs.pm/toy/) or [decode/zero](https://hexdocs.pm/decode/)\npackages.  After gathering some feedback, only one of them will be kept.\n\n\nHTML forms rendered in the browser and the data they are parsed into are\nintrinsically linked. Treating the markup and the parsing as two separate\nproblems to solve is inconvenient and leads to bugs. This library aims\nto make that link explicit and easy to manage, while making it really easy\nto make accessible forms.\n\n```sh\ngleam add formz@0.1\n```\n\n## Creating a form\n\nA `formz` form is a list of fields and a decoder function.\n\n### builder pattern\n\nWith the builder pattern, you add the fields and then explicitly specify the\ndecoder function...\n\n```gleam\nimport formz/field.{field}\nimport formz/formz_builder as formz\nimport formz_string/definitions\n\npub fn make_form() {\n  formz.new()\n  |&gt; formz.add(field(&quot;username&quot;), definitions.text_field())\n  |&gt; formz.add(field(&quot;password&quot;), definitions.password_field())\n  |&gt; formz.decodes(fn(username) { fn(password) { #(username, password) } })\n}\n```\n\n### `use`/callbacks pattern\n\nWith the `use`/callbakcks pattern, you create the decoder function as you add\nthe fields...\n\n```gleam\nimport formz/field.{field}\nimport formz/formz_use as formz\nimport formz_string/definitions\n\npub fn make_form() {\n  use username &lt;- formz.with(field(&quot;username&quot;), definitions.text_field())\n  use password &lt;- formz.with(field(&quot;password&quot;), definitions.password_field())\n\n  formz.create_form(#(username, password))\n}\n```\n\n## Creating fields\n\nThere are two parts to adding a field to a form (seen above):\n\n1. Specific, unique details about the field, such as its name, label, help text,\n   disabled/enabled state, etc.\n2. A field &quot;definition&quot; which says (A) how to generate the HTML &quot;widget&quot;\n   for the field, and (B) how to parse, or &quot;transform&quot; the data from the field. These\n   definitions are reusable and can be shared between fields, forms and projects.\n\n### Field details\n\n```gleam\n// name is required, the other details are optional\nfield(named: &quot;username&quot;)\n|&gt; field.set_label(&quot;Username&quot;)\n|&gt; field.set_help_text(&quot;Only alphanumeric characters are allowed.&quot;)\n```\n\n```gleam\nfield(named: &quot;userid&quot;) |&gt; field.make_hidden |&gt; field.set_value(&quot;42&quot;)\n```\n\n### Field definition\n\nThis library is format-agnostic and you can generate HTML widgets as raw\nstrings, Lustre elements, Nakai nodes, something else, etc. There are\ncurrently three formz libraries that provide common field definitions in\ndifferent formats.\n\n- [formz_string](https://hexdocs.pm/formz_string/)\n- [formz_nakai](https://hexdocs.pm/formz_nakai/)\n- [formz_lustre](https://hexdocs.pm/formz_lustre/) (untested in a browser,\n  would it be useful there??)\n\n\nThere is also a *simple* validation module with some examples, and to cover\nsome basics.\n\n```gleam\n/// you won&#39;t often need to do this directly (I think??).  The idea is that\n/// there&#39;d be libs with the definitions you need.\n\nimport formz/definition.{Definition}\nimport formz/field\nimport formz/validation\nimport formz/widget\nimport lustre/attribute\nimport lustre/element\nimport lustre/element/html\n\nfn password_widget(\n  field: field.Field,\n  args: widget.Args,\n) -&gt; element.Element(msg) {\n  html.input([\n    attribute.type_(&quot;password&quot;),\n    attribute.name(field.name),\n    attribute.id(args.id),\n    attribute.attribute(&quot;aria-labelledby&quot;, field.label),\n  ])\n}\n\npub fn password_field() {\n  Definition(password_widget, validation.string, &quot;&quot;)\n}\n```\n\n\n\n## Generating HTML for a form\n\nGenerally speaking, the idea with a `formz` form is that you are not going\nto generate the HTML for each field individually, but rather, you&#39;d use\na function to loop through each field, generating semantic, accessible\nmarkup for each one.\n\nThe specifics of how you would do this are going\nto vary greatly for each project and its styling/markup needs.\n\n\nHowever, the three `formz_*` libraries mentioned above all provide a\nsimple form generator function that you can use as is, or as a starting\npoint for your own.  `formz` is BYOS, Bring Your Own Stylesheet, so the\nbuilt-in form generators come unstyled. If there is interest, I could add\na super simple CSS file to get the ball rolling and make the default\nforms easier to use out of the box.\n\nThat said, you can create the form HTML yourself, directly for each field.\nThere&#39;s an example in the demo project showing how to do this.\n\n### Generating form HTML using the `formz_string` library\n\nThe built-in form generators all leave it as homework to add the form tags\nand submit buttons.\n\n```gleam\nimport formz_string/simple\n\npub fn show_form(form) -&gt; String {\n  &quot;&lt;form method=\\&quot;post\\&quot;&gt;&quot;\n  &lt;&gt; simple.generate_form(form)\n  &lt;&gt; &quot;&lt;p&gt;&lt;button type\\&quot;submit\\&quot;&gt;Submit&lt;/button&gt;&lt;/p&gt;&quot;\n  &lt;&gt; &quot;&lt;/form&gt;&quot;\n}\n```\n\n\n## Parsing form data\n\nYou can parse a `formz` form with a tuple of values and names, typically from\na POST request.  Here we parse in a `wisp` handler:\n\n```gleam\npub fn handle_form_submission(req: Request) -&gt; Response {\n  use formdata &lt;- wisp.require_form(req)\n\n  let result = make_form()\n  |&gt; formz.data(formdata.values)\n  |&gt; formz.parse\n\n  case result {\n    Ok(credentials) -&gt; {\n      let #(username, password) = credentials\n      wisp.ok()\n      |&gt; wisp.html_body(string_builder.from_string(&quot;Hello &quot;&lt;&gt;username&lt;&gt;&quot;!&quot;))\n    }\n    Error(form_with_errors) -&gt; {\n      show_form(form_with_errors)\n    }\n  }\n}\n```\n\nHowever, often you want to parse a form, and then... you know... act on that\ndata, and in doing so you might discover more errors for the form.  In this\nsituation you can use `parse_then_try`:\n\n```gleam\npub fn handle_form_submission(req: Request) -&gt; Response {\n  use formdata &lt;- wisp.require_form(req)\n\n  let result = make_form()\n  |&gt; formz.data(formdata.values)\n  |&gt; formz.parse_then_try(fn(form, credentials) {\n    case credentials {\n      #(&quot;admin&quot; as username, &quot;l33t&quot;) -&gt; Ok(username)\n      #(&quot;admin&quot;, _) -&gt;\n        form\n        |&gt; formz.update_field(&quot;password&quot;, field.set_error(_, &quot;Wrong password&quot;))\n        |&gt; Error\n      _ -&gt;\n        form\n        |&gt; formz.update_field(&quot;username&quot;, field.set_error(_, &quot;Wrong username&quot;))\n        |&gt; Error\n    }\n  })\n\n  case result {\n    Ok(username) -&gt; {\n      wisp.ok()\n      |&gt; wisp.html_body(string_builder.from_string(&quot;Hello &quot; &lt;&gt; username &lt;&gt; &quot;!&quot;))\n    }\n    Error(form_with_errors) -&gt; {\n      show_form(form_with_errors)\n    }\n  }\n}\n```\n\n## See it in action\n\nThere is a [demo wisp app](https://github.com/bentomas/formz/tree/main/formz_demo)\nshowing a few interactive examples of how `formz` works in the repo.\n","url":"index.html"},{"doc":"formz","title":"main","content":"pub fn main() -&gt; Nil\n\nSynonyms:\nformz.main\nformz main","url":"formz.html#main"},{"doc":"formz","title":"formz","content":" This will eventually be the home of `formz_builder` or `formz_use`.","url":"formz.html"},{"doc":"formz/field","title":"Field","content":"pub type Field {\n  Valid(\n    name: String,\n    label: String,\n    help_text: String,\n    disabled: Bool,\n    required: Bool,\n    hidden: Bool,\n    value: String,\n  )\n  Invalid(\n    name: String,\n    label: String,\n    help_text: String,\n    disabled: Bool,\n    required: Bool,\n    hidden: Bool,\n    value: String,\n    error: String,\n  )\n}\n\nValid(\n  name: String,\n  label: String,\n  help_text: String,\n  disabled: Bool,\n  required: Bool,\n  hidden: Bool,\n  value: String,\n)\n\n\nInvalid(\n  name: String,\n  label: String,\n  help_text: String,\n  disabled: Bool,\n  required: Bool,\n  hidden: Bool,\n  value: String,\n  error: String,\n)\n\n\nSynonyms:\nformz/field.Field\nformz/field Field","url":"formz/field.html#Field"},{"doc":"formz/field","title":"field","content":"pub fn field(named name: String) -&gt; Field\n\nSynonyms:\nformz/field.field\nformz/field field","url":"formz/field.html#field"},{"doc":"formz/field","title":"make_disabled","content":"pub fn make_disabled(field: Field) -&gt; Field\n\nSynonyms:\nformz/field.make_disabled\nformz/field make_disabled","url":"formz/field.html#make_disabled"},{"doc":"formz/field","title":"make_enabled","content":"pub fn make_enabled(field: Field) -&gt; Field\n\nSynonyms:\nformz/field.make_enabled\nformz/field make_enabled","url":"formz/field.html#make_enabled"},{"doc":"formz/field","title":"make_hidden","content":"pub fn make_hidden(field: Field) -&gt; Field\n\nSynonyms:\nformz/field.make_hidden\nformz/field make_hidden","url":"formz/field.html#make_hidden"},{"doc":"formz/field","title":"make_visible","content":"pub fn make_visible(field: Field) -&gt; Field\n\nSynonyms:\nformz/field.make_visible\nformz/field make_visible","url":"formz/field.html#make_visible"},{"doc":"formz/field","title":"set_error","content":"pub fn set_error(field: Field, error: String) -&gt; Field\n\nSynonyms:\nformz/field.set_error\nformz/field set_error","url":"formz/field.html#set_error"},{"doc":"formz/field","title":"set_help_text","content":"pub fn set_help_text(field: Field, help_text: String) -&gt; Field\n\nSynonyms:\nformz/field.set_help_text\nformz/field set_help_text","url":"formz/field.html#set_help_text"},{"doc":"formz/field","title":"set_label","content":"pub fn set_label(field: Field, label: String) -&gt; Field\n\nSynonyms:\nformz/field.set_label\nformz/field set_label","url":"formz/field.html#set_label"},{"doc":"formz/field","title":"set_name","content":"pub fn set_name(field: Field, name: String) -&gt; Field\n\nSynonyms:\nformz/field.set_name\nformz/field set_name","url":"formz/field.html#set_name"},{"doc":"formz/field","title":"set_string_value","content":"pub fn set_string_value(field: Field, value: String) -&gt; Field\n\nSynonyms:\nformz/field.set_string_value\nformz/field set_string_value","url":"formz/field.html#set_string_value"},{"doc":"formz/field","title":"formz/field","content":"","url":"formz/field.html"},{"doc":"formz/widget","title":"Args","content":"pub type Args {\n  Args(\n    id: String,\n    labelled_by: LabelledBy,\n    described_by: DescribedBy,\n  )\n}\n\nArgs(\n  id: String,\n  labelled_by: LabelledBy,\n  described_by: DescribedBy,\n)\n\n\nSynonyms:\nformz/widget.Args\nformz/widget Args","url":"formz/widget.html#Args"},{"doc":"formz/widget","title":"DescribedBy","content":"pub type DescribedBy {\n  DescribedByElementWithId(id: String)\n  DescribedByNone\n}\n\nDescribedByElementWithId(id: String)\n\n\nDescribedByNone\n\n\nSynonyms:\nformz/widget.DescribedBy\nformz/widget DescribedBy","url":"formz/widget.html#DescribedBy"},{"doc":"formz/widget","title":"LabelledBy","content":"pub type LabelledBy {\n  LabelledByLabelFor\n  LabelledByFieldValue\n  LabelledByElementWithId(id: String)\n}\n\nLabelledByLabelFor\n\n\nLabelledByFieldValue\n\n\nLabelledByElementWithId(id: String)\n\n\nSynonyms:\nformz/widget.LabelledBy\nformz/widget LabelledBy","url":"formz/widget.html#LabelledBy"},{"doc":"formz/widget","title":"Widget","content":"pub type Widget(format) =\n  fn(field.Field, Args) -&gt; format\n\n\nSynonyms:\nformz/widget.Widget\nformz/widget Widget","url":"formz/widget.html#Widget"},{"doc":"formz/widget","title":"formz/widget","content":"","url":"formz/widget.html"},{"doc":"formz/definition","title":"Definition","content":"pub type Definition(format, output) {\n  Definition(\n    widget: widget.Widget(format),\n    transform: fn(String) -&gt; Result(output, String),\n    placeholder: output,\n  )\n}\n\nDefinition(\n  widget: widget.Widget(format),\n  transform: fn(String) -&gt; Result(output, String),\n  placeholder: output,\n)\n\n\nSynonyms:\nformz/definition.Definition\nformz/definition Definition","url":"formz/definition.html#Definition"},{"doc":"formz/definition","title":"set_widget","content":"pub fn set_widget(\n  kind: Definition(a, b),\n  widget: fn(Field, Args) -&gt; a,\n) -&gt; Definition(a, b)\n\nSynonyms:\nformz/definition.set_widget\nformz/definition set_widget","url":"formz/definition.html#set_widget"},{"doc":"formz/definition","title":"transforms","content":"pub fn transforms(\n  kind: Definition(a, b),\n  placeholder: c,\n  next: fn(b) -&gt; Result(c, String),\n) -&gt; Definition(a, c)\n\nSynonyms:\nformz/definition.transforms\nformz/definition transforms","url":"formz/definition.html#transforms"},{"doc":"formz/definition","title":"validates","content":"pub fn validates(\n  kind: Definition(a, b),\n  next: fn(b) -&gt; Result(b, String),\n) -&gt; Definition(a, b)\n\nSynonyms:\nformz/definition.validates\nformz/definition validates","url":"formz/definition.html#validates"},{"doc":"formz/definition","title":"formz/definition","content":"","url":"formz/definition.html"},{"doc":"formz/form_details","title":"FormDetails","content":"pub type FormDetails {\n  FormDetails(\n    name: String,\n    label: String,\n    help_text: String,\n    disabled: Bool,\n  )\n}\n\nFormDetails(\n  name: String,\n  label: String,\n  help_text: String,\n  disabled: Bool,\n)\n\n\nSynonyms:\nformz/form_details.FormDetails\nformz/form_details FormDetails","url":"formz/form_details.html#FormDetails"},{"doc":"formz/form_details","title":"form_details","content":"pub fn form_details(name: String) -&gt; FormDetails\n\nSynonyms:\nformz/form_details.form_details\nformz/form_details form_details","url":"formz/form_details.html#form_details"},{"doc":"formz/form_details","title":"make_disabled","content":"pub fn make_disabled(sub: FormDetails) -&gt; FormDetails\n\nSynonyms:\nformz/form_details.make_disabled\nformz/form_details make_disabled","url":"formz/form_details.html#make_disabled"},{"doc":"formz/form_details","title":"make_enabled","content":"pub fn make_enabled(sub: FormDetails) -&gt; FormDetails\n\nSynonyms:\nformz/form_details.make_enabled\nformz/form_details make_enabled","url":"formz/form_details.html#make_enabled"},{"doc":"formz/form_details","title":"set_help_text","content":"pub fn set_help_text(\n  sub: FormDetails,\n  help_text: String,\n) -&gt; FormDetails\n\nSynonyms:\nformz/form_details.set_help_text\nformz/form_details set_help_text","url":"formz/form_details.html#set_help_text"},{"doc":"formz/form_details","title":"set_label","content":"pub fn set_label(sub: FormDetails, label: String) -&gt; FormDetails\n\nSynonyms:\nformz/form_details.set_label\nformz/form_details set_label","url":"formz/form_details.html#set_label"},{"doc":"formz/form_details","title":"set_name","content":"pub fn set_name(sub: FormDetails, name: String) -&gt; FormDetails\n\nSynonyms:\nformz/form_details.set_name\nformz/form_details set_name","url":"formz/form_details.html#set_name"},{"doc":"formz/form_details","title":"formz/form_details","content":"","url":"formz/form_details.html"},{"doc":"formz/formz_builder","title":"Form","content":"pub opaque type Form(format, output, decoder, has_decoder)\n\n\nSynonyms:\nformz/formz_builder.Form\nformz/formz_builder Form","url":"formz/formz_builder.html#Form"},{"doc":"formz/formz_builder","title":"FormItem","content":"pub type FormItem(format) {\n  Element(field.Field, widget: widget.Widget(format))\n  Set(form_details.FormDetails, items: List(FormItem(format)))\n}\n\nElement(field.Field, widget: widget.Widget(format))\n\n\nSet(form_details.FormDetails, items: List(FormItem(format)))\n\n\nSynonyms:\nformz/formz_builder.FormItem\nformz/formz_builder FormItem","url":"formz/formz_builder.html#FormItem"},{"doc":"formz/formz_builder","title":"HasDecoder","content":"pub type HasDecoder\n\n\nSynonyms:\nformz/formz_builder.HasDecoder\nformz/formz_builder HasDecoder","url":"formz/formz_builder.html#HasDecoder"},{"doc":"formz/formz_builder","title":"NoDecoder","content":"pub type NoDecoder\n\n\nSynonyms:\nformz/formz_builder.NoDecoder\nformz/formz_builder NoDecoder","url":"formz/formz_builder.html#NoDecoder"},{"doc":"formz/formz_builder","title":"add","content":"pub fn add(\n  previous_form: Form(a, fn(b) -&gt; c, d, e),\n  field: Field,\n  definition: Definition(a, b),\n) -&gt; Form(a, c, d, e)\n\nSynonyms:\nformz/formz_builder.add\nformz/formz_builder add","url":"formz/formz_builder.html#add"},{"doc":"formz/formz_builder","title":"add_form","content":"pub fn add_form(\n  previous_form: Form(a, fn(b) -&gt; c, d, e),\n  details: FormDetails,\n  sub: Form(a, b, f, HasDecoder),\n) -&gt; Form(a, c, d, e)\n\nSynonyms:\nformz/formz_builder.add_form\nformz/formz_builder add_form","url":"formz/formz_builder.html#add_form"},{"doc":"formz/formz_builder","title":"data","content":"pub fn data(\n  form: Form(a, b, c, d),\n  input_data: List(#(String, String)),\n) -&gt; Form(a, b, c, d)\n\nSynonyms:\nformz/formz_builder.data\nformz/formz_builder data","url":"formz/formz_builder.html#data"},{"doc":"formz/formz_builder","title":"decodes","content":"pub fn decodes(\n  form: Form(a, b, c, d),\n  decoder: c,\n) -&gt; Form(a, b, c, HasDecoder)\n\nSynonyms:\nformz/formz_builder.decodes\nformz/formz_builder decodes","url":"formz/formz_builder.html#decodes"},{"doc":"formz/formz_builder","title":"get","content":"pub fn get(\n  form: Form(a, b, c, d),\n  name: String,\n) -&gt; Result(FormItem(a), Nil)\n\nSynonyms:\nformz/formz_builder.get\nformz/formz_builder get","url":"formz/formz_builder.html#get"},{"doc":"formz/formz_builder","title":"items","content":"pub fn items(form: Form(a, b, c, d)) -&gt; List(FormItem(a))\n\nSynonyms:\nformz/formz_builder.items\nformz/formz_builder items","url":"formz/formz_builder.html#items"},{"doc":"formz/formz_builder","title":"new","content":"pub fn new() -&gt; Form(a, b, b, NoDecoder)\n\nSynonyms:\nformz/formz_builder.new\nformz/formz_builder new","url":"formz/formz_builder.html#new"},{"doc":"formz/formz_builder","title":"parse","content":"pub fn parse(\n  form: Form(a, b, c, HasDecoder),\n) -&gt; Result(b, Form(a, b, c, HasDecoder))\n\nSynonyms:\nformz/formz_builder.parse\nformz/formz_builder parse","url":"formz/formz_builder.html#parse"},{"doc":"formz/formz_builder","title":"parse_then_try","content":"pub fn parse_then_try(\n  form: Form(a, b, c, HasDecoder),\n  apply fun: fn(Form(a, b, c, HasDecoder), b) -&gt;\n    Result(d, Form(a, b, c, HasDecoder)),\n) -&gt; Result(d, Form(a, b, c, HasDecoder))\n\nSynonyms:\nformz/formz_builder.parse_then_try\nformz/formz_builder parse_then_try","url":"formz/formz_builder.html#parse_then_try"},{"doc":"formz/formz_builder","title":"update","content":"pub fn update(\n  form: Form(a, b, c, d),\n  name: String,\n  fun: fn(FormItem(a)) -&gt; FormItem(a),\n) -&gt; Form(a, b, c, e)\n\nSynonyms:\nformz/formz_builder.update\nformz/formz_builder update","url":"formz/formz_builder.html#update"},{"doc":"formz/formz_builder","title":"update_field","content":"pub fn update_field(\n  form: Form(a, b, c, d),\n  name: String,\n  fun: fn(Field) -&gt; Field,\n) -&gt; Form(a, b, c, d)\n\nSynonyms:\nformz/formz_builder.update_field\nformz/formz_builder update_field","url":"formz/formz_builder.html#update_field"},{"doc":"formz/formz_builder","title":"update_subform","content":"pub fn update_subform(\n  form: Form(a, b, c, d),\n  name: String,\n  fun: fn(FormDetails) -&gt; FormDetails,\n) -&gt; Form(a, b, c, d)\n\nSynonyms:\nformz/formz_builder.update_subform\nformz/formz_builder update_subform","url":"formz/formz_builder.html#update_subform"},{"doc":"formz/formz_builder","title":"formz/formz_builder","content":"","url":"formz/formz_builder.html"},{"doc":"formz/formz_use","title":"Form","content":"pub opaque type Form(format, output)\n\n\nSynonyms:\nformz/formz_use.Form\nformz/formz_use Form","url":"formz/formz_use.html#Form"},{"doc":"formz/formz_use","title":"FormItem","content":"pub type FormItem(format) {\n  Element(field.Field, widget: widget.Widget(format))\n  Set(form_details.FormDetails, items: List(FormItem(format)))\n}\n\nElement(field.Field, widget: widget.Widget(format))\n\n\nSet(form_details.FormDetails, items: List(FormItem(format)))\n\n\nSynonyms:\nformz/formz_use.FormItem\nformz/formz_use FormItem","url":"formz/formz_use.html#FormItem"},{"doc":"formz/formz_use","title":"create_form","content":"pub fn create_form(thing: a) -&gt; Form(b, a)\n\nSynonyms:\nformz/formz_use.create_form\nformz/formz_use create_form","url":"formz/formz_use.html#create_form"},{"doc":"formz/formz_use","title":"data","content":"pub fn data(\n  form: Form(a, b),\n  input_data: List(#(String, String)),\n) -&gt; Form(a, b)\n\nSynonyms:\nformz/formz_use.data\nformz/formz_use data","url":"formz/formz_use.html#data"},{"doc":"formz/formz_use","title":"get","content":"pub fn get(\n  form: Form(a, b),\n  name: String,\n) -&gt; Result(FormItem(a), Nil)\n\nSynonyms:\nformz/formz_use.get\nformz/formz_use get","url":"formz/formz_use.html#get"},{"doc":"formz/formz_use","title":"items","content":"pub fn items(form: Form(a, b)) -&gt; List(FormItem(a))\n\nSynonyms:\nformz/formz_use.items\nformz/formz_use items","url":"formz/formz_use.html#items"},{"doc":"formz/formz_use","title":"parse","content":"pub fn parse(form: Form(a, b)) -&gt; Result(b, Form(a, b))\n\nSynonyms:\nformz/formz_use.parse\nformz/formz_use parse","url":"formz/formz_use.html#parse"},{"doc":"formz/formz_use","title":"parse_then_try","content":"pub fn parse_then_try(\n  form: Form(a, b),\n  apply fun: fn(Form(a, b), b) -&gt; Result(c, Form(a, b)),\n) -&gt; Result(c, Form(a, b))\n\nSynonyms:\nformz/formz_use.parse_then_try\nformz/formz_use parse_then_try","url":"formz/formz_use.html#parse_then_try"},{"doc":"formz/formz_use","title":"update","content":"pub fn update(\n  form: Form(a, b),\n  name: String,\n  fun: fn(FormItem(a)) -&gt; FormItem(a),\n) -&gt; Form(a, b)\n\nSynonyms:\nformz/formz_use.update\nformz/formz_use update","url":"formz/formz_use.html#update"},{"doc":"formz/formz_use","title":"update_field","content":"pub fn update_field(\n  form: Form(a, b),\n  name: String,\n  fun: fn(Field) -&gt; Field,\n) -&gt; Form(a, b)\n\nSynonyms:\nformz/formz_use.update_field\nformz/formz_use update_field","url":"formz/formz_use.html#update_field"},{"doc":"formz/formz_use","title":"update_subform","content":"pub fn update_subform(\n  form: Form(a, b),\n  name: String,\n  fun: fn(FormDetails) -&gt; FormDetails,\n) -&gt; Form(a, b)\n\nSynonyms:\nformz/formz_use.update_subform\nformz/formz_use update_subform","url":"formz/formz_use.html#update_subform"},{"doc":"formz/formz_use","title":"with","content":"pub fn with(\n  field: Field,\n  is definition: Definition(a, b),\n  rest fun: fn(b) -&gt; Form(a, c),\n) -&gt; Form(a, c)\n\nSynonyms:\nformz/formz_use.with\nformz/formz_use with","url":"formz/formz_use.html#with"},{"doc":"formz/formz_use","title":"with_form","content":"pub fn with_form(\n  details: FormDetails,\n  sub: Form(a, b),\n  fun: fn(b) -&gt; Form(a, c),\n) -&gt; Form(a, c)\n\nSynonyms:\nformz/formz_use.with_form\nformz/formz_use with_form","url":"formz/formz_use.html#with_form"},{"doc":"formz/formz_use","title":"formz/formz_use","content":"","url":"formz/formz_use.html"},{"doc":"formz/validation","title":"and","content":"pub fn and(\n  previous: fn(a) -&gt; Result(b, String),\n  next: fn(b) -&gt; Result(c, String),\n) -&gt; fn(a) -&gt; Result(c, String)\n\nSynonyms:\nformz/validation.and\nformz/validation and","url":"formz/validation.html#and"},{"doc":"formz/validation","title":"boolean","content":"pub fn boolean(str: String) -&gt; Result(Bool, String)\n\nSynonyms:\nformz/validation.boolean\nformz/validation boolean","url":"formz/validation.html#boolean"},{"doc":"formz/validation","title":"email","content":"pub fn email(input: String) -&gt; Result(String, String)\n\nSynonyms:\nformz/validation.email\nformz/validation email","url":"formz/validation.html#email"},{"doc":"formz/validation","title":"enum","content":"pub fn enum(\n  variants: List(#(String, a)),\n) -&gt; fn(String) -&gt; Result(a, String)\n\nSynonyms:\nformz/validation.enum\nformz/validation enum","url":"formz/validation.html#enum"},{"doc":"formz/validation","title":"enum_by_index","content":"pub fn enum_by_index(\n  variants: List(#(String, a)),\n) -&gt; fn(String) -&gt; Result(a, String)\n\nSynonyms:\nformz/validation.enum_by_index\nformz/validation enum_by_index","url":"formz/validation.html#enum_by_index"},{"doc":"formz/validation","title":"int","content":"pub fn int(str: String) -&gt; Result(Int, String)\n\nSynonyms:\nformz/validation.int\nformz/validation int","url":"formz/validation.html#int"},{"doc":"formz/validation","title":"must_be_longer_than","content":"pub fn must_be_longer_than(\n  length: Int,\n) -&gt; fn(String) -&gt; Result(String, String)\n\nSynonyms:\nformz/validation.must_be_longer_than\nformz/validation must_be_longer_than","url":"formz/validation.html#must_be_longer_than"},{"doc":"formz/validation","title":"number","content":"pub fn number(str: String) -&gt; Result(Float, String)\n\nSynonyms:\nformz/validation.number\nformz/validation number","url":"formz/validation.html#number"},{"doc":"formz/validation","title":"or","content":"pub fn or(\n  previous: fn(a) -&gt; Result(b, String),\n  next: fn(a) -&gt; Result(b, String),\n) -&gt; fn(a) -&gt; Result(b, String)\n\nSynonyms:\nformz/validation.or\nformz/validation or","url":"formz/validation.html#or"},{"doc":"formz/validation","title":"replace_error","content":"pub fn replace_error(\n  previous: fn(a) -&gt; Result(b, String),\n  error: String,\n) -&gt; fn(a) -&gt; Result(b, String)\n\nSynonyms:\nformz/validation.replace_error\nformz/validation replace_error","url":"formz/validation.html#replace_error"},{"doc":"formz/validation","title":"string","content":"pub fn string(str: String) -&gt; Result(String, String)\n\nSynonyms:\nformz/validation.string\nformz/validation string","url":"formz/validation.html#string"},{"doc":"formz/validation","title":"true","content":"pub fn true(str: String) -&gt; Result(Bool, String)\n\nSynonyms:\nformz/validation.true\nformz/validation true","url":"formz/validation.html#true"},{"doc":"formz/validation","title":"formz/validation","content":"","url":"formz/validation.html"}]);